@@ -105,7 +105,7 @@ check.
 %!  list_undefined(+Options) is det.
 %
 %   Report undefined predicates.  This   predicate  finds  undefined
-%   predciates by decompiling and analyzing the body of all clauses.
+%   predicates by decompiling and analyzing the body of all clauses.
 %   Options:
 %
 %       * module_class(+Classes)
@@ -164,6 +164,7 @@ compiled(system:'$cut'/0).
 compiled(system:'$reset'/0).
 compiled(system:'$call_continuation'/1).
 compiled(system:'$shift'/1).
+compiled('$engines':'$yield'/0).
 
 goal_pi(M:Head, M:Name/Arity) :-
     functor(Head, Name, Arity).
