@@ -6,6 +6,8 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import java.util.NoSuchElementException;
+
 import org.jpl7.fli.Prolog;
 import org.jpl7.fli.engine_t;
 import org.jpl7.fli.fid_t;
@@ -13,6 +15,10 @@ import org.jpl7.fli.predicate_t;
 import org.jpl7.fli.qid_t;
 import org.jpl7.fli.term_t;
 
+import java.util.logging.Level;
+import java.util.logging.Logger;
+
+
 /**
  * A Query instance is created by an application in order to query the Prolog
  * database (or to invoke a built-in predicate). It is initialised with a
@@ -88,11 +94,15 @@ import org.jpl7.fli.term_t;
  * POSSIBILITY OF SUCH DAMAGE.
  * <hr>
  *
- * @author Fred Dushin <fadushin@syr.edu>
+ * @author Fred Dushin fadushin@syr.edu
  * @version $Revision$
  */
 public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String, Term>> { // was
 																							// Enumeration<Object>
+
+    private static final Logger LOGGER = Logger.getLogger( Query.class.getName() );
+    private static final Level levelLog = Level.INFO;
+
 	/**
 	 * the Compound or Atom (but not Dict, Float, Integer or Variable)
 	 * corresponding to the goal of this Query
@@ -113,6 +123,15 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		return goal_;
 	}
 
+	/**
+	 * Returns the context module for this Query
+	 *
+	 * @return a String representing the context in which the goal will be run
+	 */
+	public final String getContext() { return contextModule; }
+
+
+
 	/**
 	 * This constructor creates a Query whose goal is the specified Term. The
 	 * Query is initially closed. <b>NB</b> Creating an instance of the Query
@@ -120,19 +139,13 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * can be a Compound or an Atom (Atom extends Compound), but cannot be an
 	 * instance of jpl.Float, jpl.Integer or jpl.Variable.
 	 *
-	 * @param t
-	 *            the goal of this Query
+	 * @param t the goal of this Query; must be Atom or Compound
+         * @throws JPLException  if term provided is not of right sort Atom or Compound
 	 */
-	public Query(Term t) { // formerly insisted (confusingly) on a Compound (or
-							// Atom)
-		this.goal_ = Query1(t);
-	}
-
-	private Term Query1(Term t) {
-		if (t instanceof Atom) {
-			return t;
-		} else if (t instanceof Compound) {
-			return t;
+	public Query(Term t) {
+	        LOGGER.setLevel(levelLog);  // Set the logging level for this query
+		if ((t instanceof Atom) || (t instanceof Compound)) {
+            		this.goal_ = t;
 		} else if (t instanceof Float) {
 			throw new JPLException("a Query's goal must be an Atom or Compound (not a Float)");
 		} else if (t instanceof Integer) {
@@ -144,12 +157,25 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		}
 	}
 
-	/**
+    /**
+     * This constructor builds a Query from the given Prolog source text, "as is".
+     * Throws PrologException containing error(syntax_error(_),_) if text is invalid.
+     *
+     * @param text the complete Prolog source text of this Query
+     * @throws PrologException containing error(syntax_error(_),_) if text is invalid as a term.
+     * @throws JPLException  if term provided is not of right sort Atom or Compound
+     */
+    public Query(String text) {
+        this(Util.textToTerm(text));
+    }
+
+
+    /**
 	 * If text denotes an atom, this constructor is shorthand for
-	 * <font face="monospace">new Query(new Compound(name,args))</font>, but if
-	 * text denotes a term containing N query (?) symbols and there are N args,
-	 * each query is replaced by its corresponding arg to provide the new
-	 * Query's goal.
+	 * new Query(new Compound(name,args)).
+	 *
+	 * If text denotes a term containing N placehholder symbols ? and there are N args,
+	 * each ? is replaced by its corresponding arg to provide the new Query's goal.
 	 *
 	 * @param text
 	 *            the name of the principal functor of this Query's goal
@@ -157,15 +183,28 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *            the arguments of this Query's goal
 	 */
 	public Query(String text, Term[] args) {
-		this(Query1(text, args));
+		this(buildQueryTerm(text, args));
 	}
 
-	// convenience case for a single arg
+	// Just a convenience  shortcut case for a single arg; will build a monadic query "text(arg)"
 	public Query(String text, Term arg) {
-		this(Query1(text, new Term[] { arg }));
+		this(text, new Term[] { arg });
 	}
 
-	private static Term Query1(String text, Term[] args) {
+
+    /**
+     * Builds a Term representing a query
+     *
+     * If text denotes an atom, it builds a Term of the form text(args)
+     * If text denotes a compound term containing N placehholder symbols ?, then args
+     * will be injected sequentially (has to be same number of ? as args)
+     *
+     * @param text
+     *            the name of the principal functor of this Query's goal
+     * @param args
+     *            the arguments of this Query's goal
+     */
+	private static Term buildQueryTerm(String text, Term[] args) {
 		Term t = Util.textToTerm(text);
 		if (t instanceof Atom) {
 			return new Compound(text, args);
@@ -174,18 +213,15 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		}
 	}
 
-	/**
-	 * This constructor builds a Query from the given Prolog source text. Throws
-	 * PrologException containing error(syntax_error(_),_) if text is invalid.
-	 *
-	 * @param text
-	 *            the Prolog source text of this Query
-	 */
-	public Query(String text) {
-		this(Util.textToTerm(text));
-	}
+
+
+
+
+
+
 
 	/**
+         * This method is required by Iterator interface
 	 * a Query is its own Iterator
 	 *
 	 * @see java.lang.Iterable#iterator()
@@ -194,21 +230,19 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		return this;
 	}
 
-	/**
-	 * whether this Query has a (further) solution
-	 *
-	 * @see java.util.Iterator#hasNext()
-	 */
+    /**
+     * This method is required by Iterator interface
+     * It is a wrapper for {@link #hasMoreSolutions()}
+     *
+     * @see java.util.Iterator#hasNext()
+     */
 	public boolean hasNext() {
-		if (!open) { // lazily open the Query, enabling it to be its own
-						// iterator
-			open();
-		}
-		return get1();
+	    return hasMoreSolutions();
 	}
 
 	/**
-	 * this Query's next solution
+         * This method is required by Iterator interface
+         * It is a wrapper for {@link #nextSolution()}
 	 *
 	 * @see java.util.Iterator#next()
 	 */
@@ -217,7 +251,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	}
 
 	/**
-	 * this method (required by Iterator interface) is a no-op
+	 * This method (required by Iterator interface) is a no-op
 	 *
 	 * @see java.util.Iterator#remove()
 	 */
@@ -225,7 +259,30 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		// no op
 	}
 
-	/**
+    /**
+     * This method implements part of the java.util.Enumeration interface.
+     * It is a wrapper for {@link #hasMoreSolutions()}.
+     *
+     * @return true if the Prolog query yields a (or another) solution, else
+     *         false.
+     */
+    public final boolean hasMoreElements() {
+        return hasMoreSolutions();
+    }
+
+    /**
+     * This method implements part of the java.util.Enumeration interface.
+     * It is a wrapper for {@link #nextSolution()}
+     * <p>
+     *
+     * @return A Map representing a substitution.
+     */
+    public final Object nextElement() {
+        return nextSolution();
+    }
+
+
+    /**
 	 * These variables are used and set across the hasMoreElements and
 	 * nextElement Enumeration interface implementation
 	 */
@@ -242,9 +299,11 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 									// else undefined
 	private qid_t qid = null; // id of current Prolog query iff open, else null
 
-	//
+        private Boolean hasNextSolution = null; // is there a next solution? null means "we do not know yet, haven't fetch"
+
+
 	/**
-	 * isOpen() returns true iff the query is open.
+	 * Whether the query is open.
 	 *
 	 * @return true if the query is open, otherwise false.
 	 */
@@ -253,6 +312,14 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	}
 
 	/**
+	* Returns the engine attached to this query
+	*
+	* @return a long number representing the id of the SWI Prolog engine used in this query
+	*/
+	public final long getEngine() { return engine.value; }
+
+
+        /**
 	 * This method returns true if JPL was able to initiate a "call" of this
 	 * Query within a Prolog engine. It is designed to be used with the
 	 * nextSolution() method to retrieve one or more substitutions in the form
@@ -269,10 +336,11 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * @return true if the Prolog query succeeds; otherwise false.
 	 */
 	public final boolean hasMoreSolutions() {
-		if (!open) {
-			open();
-		}
-		return get1();
+		if (hasNextSolution == null) {
+	            if (!open) open();
+	            hasNextSolution = fetchNextSolution();
+	        }
+	        return hasNextSolution;
 	}
 
 	/**
@@ -337,9 +405,26 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		qid = Prolog.open_query(Prolog.new_module(Prolog.new_atom(module)), Prolog.Q_CATCH_EXCEPTION, predicate,
 				term0);
 		open = true;
+		hasNextSolution = null; // we have not fetch yet any solution
 	}
 
-	private final boolean get1() { // try to get the next solution; if none,
+    /**
+     * Reset the query to its start: closed and no solution found so far. This will allow a query to re-start
+     *
+     */
+    public final void reset() {
+        close();
+        hasNextSolution = null;
+    }
+
+    /**
+     * Tell Prolog engine to fetch the next solution for the current active query (like hitting ;)
+     * If there are no more solutions, then just close the query
+     *
+     * @return whether a new solutions was found or there are no more solutions
+     * @throws PrologException with the term of the error from Prolog (e.g., syntax error in query or non existence of predicates)
+     */
+	private final boolean fetchNextSolution() { // try to get the next solution; if none,
 									// close the query;
 		if (Prolog.next_solution(qid)) {
 			return true;
@@ -358,52 +443,21 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	}
 
 	/**
-	 * This method returns a java.util.Map, which represents a set of bindings
-	 * from the names of query variables to terms within the solution.
-	 * <p>
-	 * For example, if a Query has an occurrence of a jpl.Variable, say, named
-	 * "X", one can obtain the Term bound to "X" in the solution by looking up
-	 * "X" in the Map.
-	 *
-	 * <pre>
-	 * Variable x = new Variable("X");
-	 * Query q = // obtain Query reference (with x in the Term array)
-	 * while (q.hasMoreSolutions()) {
-	 *     Map solution = q.nextSolution();
-	 *     // make t the Term bound to "X" in the solution
-	 *     Term t = (Term) solution.get("X");
-	 *     // ...
-	 * }
-	 * </pre>
-	 *
-	 * Programmers should obey the following rules when using this method.
-	 * <menu>
-	 * <li>The nextSolution() method should only be called after the
-	 * hasMoreSolutions() method returns true; otherwise a JPLException will be
-	 * raised, indicating that the Query is no longer open.
-	 * <li>The nextSolution() and hasMoreSolutions() should be called in the
-	 * same thread of execution, for a given Query instance. </menu>
-	 *
-	 * This method will throw a JPLException if Query is not open.
-	 *
-	 * @return A Map representing a substitution, or null
+     * Returns the next solution of the query
+     *
+     * @return the binding representing the next solution
+	 * @deprecated use nextSolution()
 	 */
+	@Deprecated
 	public final Map<String, Term> getSolution() {
-		// oughta check: thread has query's engine
-		if (!open) {
-			throw new JPLException("Query is not open");
-		} else if (get1()) {
-			return get2();
-		} else {
-			return null;
-		}
+	    return nextSolution();
 	}
 
 	public final Map<String, Term> getSubstWithNameVars() {
 		// oughta check: thread has query's engine
 		if (!open) {
 			throw new JPLException("Query is not open");
-		} else if (get1()) {
+		} else if (fetchNextSolution()) {
 			return get2WithNameVars();
 		} else {
 			return null;
@@ -412,7 +466,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 
 	/**
 	 * This method returns a java.util.Map, which represents a binding from the
-	 * names of query variables to terms within the solution.
+	 * names of query variables to terms within the next solution.
 	 * <p>
 	 * For example, if a Query has an occurrence of a jpl.Variable, say, named
 	 * "X", one can obtain the Term bound to "X" in the solution by looking up
@@ -429,38 +483,29 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * }
 	 * </pre>
 	 *
-	 * Programmers should obey the following rules when using this method.
-	 * <menu>
-	 * <li>The nextSolution() method should only be called after the
-	 * hasMoreSolutions() method returns true; otherwise a JPLException will be
-	 * raised, indicating that the Query is no longer open.
-	 * <li>The nextSolution() and hasMoreSolutions() should be called in the
-	 * same thread of execution, for a given Query instance. </menu>
+	 * Programmers should be careful to call this method after checking that
+	 * there is indeed a solution, via method hasMoreSolutions().
 	 *
-	 * This method will throw a JPLException if Query is not open.
-	 *
-	 * @return A Map representing a substitution.
+	 *  @return A Map representing a substitution of the next solution.
+	 *  @throws JPLException if Query is not open.
+	 *  @throws NoSuchElementException if there are no more new solutions.
 	 */
 	public final Map<String, Term> nextSolution() {
-		return get2();
+        if (hasMoreSolutions()) {
+            hasNextSolution = null; // we reset this as we moved the pointer (we now do not know if there is a new one)
+            return getCurrentSolutionBindings();
+        } else
+            throw new NoSuchElementException("Query has already yielded all solutions");
 	}
 
-	private final Map<String, Term> get2() {
+	private final Map<String, Term> getCurrentSolutionBindings() {
 		if (!open) {
-			throw new JPLException("Query is not open");
+			throw new JPLException("Query is not open, cannot retrive solution bindings.");
 		} else {
 			Map<String, Term> substitution = new HashMap<String, Term>();
-			Term.getSubsts(substitution, new HashMap<term_t, Variable>(), goal_.args()); // NB
-																							// I
-																							// reckon
-																							// getSubsts
-																							// needn't
-																							// be
-																							// in
-																							// Term
-																							// (but
-																							// where
-																							// else?)
+			// TODO: getSubsts is in Term class, should it be there? Otherwise, where else?
+			Term.getSubsts(substitution, new HashMap<term_t, Variable>(), goal_.args());
+
 			return substitution;
 		}
 	}
@@ -488,28 +533,6 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 		}
 	}
 
-	/**
-	 * This method implements part of the java.util.Enumeration interface. It is
-	 * a wrapper for hasMoreSolutions.
-	 *
-	 * @return true if the Prolog query yields a (or another) solution, else
-	 *         false.
-	 */
-	public final boolean hasMoreElements() {
-		return hasMoreSolutions();
-	}
-
-	/**
-	 * This method implements part of the java.util.Enumeration interface. It is
-	 * a wrapper for nextSolution.
-	 * <p>
-	 *
-	 * @return A Map representing a substitution.
-	 */
-	public final Object nextElement() {
-		return nextSolution();
-	}
-
 	/**
 	 * This method can be used to close an open query before its solutions are
 	 * exhausted. It is called automatically when solutions are exhausted.
@@ -525,7 +548,6 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * Map&lt;String, Term&gt; sub3 = q.nextSolution();
 	 * q.close();
 	 * </pre>
-	 * <p>
 	 */
 	public final void close() {
 		if (!open) {
@@ -560,9 +582,8 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 			// System.out.println("JPL retaining engine[" + engine.value + "]
 		}
 		open = false; // this Query is now closed
-		engine = null; // this Query, being closed, is no longer associated with
-						// any Prolog engine
-	}
+		engine = null; // this Query, being closed, is no longer associated with any Prolog engine
+    }
 
 	/**
 	 * calls the Query's goal to exhaustion and returns an array of zero or more
@@ -581,7 +602,6 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *         Variable instances; in JPL 2.x onwards they are keyed by the
 	 *         (String) names of variables, which is consistent with the Term
 	 *         type being just a concrete syntax for terms (and hence queries).
-	 *         <p>
 	 */
 	public final Map<String, Term>[] allSolutions() {
 		if (open) {
@@ -677,6 +697,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * Maps of variablename-to-term bindings) every found solution (in the order
 	 * in which they were found).
 	 *
+     * @param n the number of solutions to cover
 	 * @return an array of Maps (possibly none), each of which is a solution (in
 	 *         the order in which they were found) of the Query; at most 'n'
 	 *         solutions will be found and returned. <b>NB</b> in JPL 1.0.1,
@@ -689,7 +710,6 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *         Variable instances; in JPL 2.x onwards they are keyed by the
 	 *         (String) names of variables, which is consistent with the Term
 	 *         type being just a concrete syntax for terms (and hence queries).
-	 *         <p>
 	 */
 	public final Map<String, Term>[] nSolutions(long n) {
 		if (open) {
@@ -724,6 +744,8 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *
 	 * @param goal
 	 *            the goal of this Query
+     * @param n the number of solutions to cover
+     * @return an array of up to the the first n binding solutions
 	 */
 	public static final Map<String, Term>[] nSolutions(Term goal, long n) {
 		return (new Query(goal)).nSolutions(n);
@@ -739,6 +761,8 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *
 	 * @param text
 	 *            a Prolog source text fragment denoting a goal
+     * @param n the number of solutions to cover
+     * @return an array of up to the the first n binding solutions
 	 */
 	public static final Map<String, Term>[] nSolutions(String text, long n) {
 		return (new Query(text)).nSolutions(n);
@@ -764,6 +788,8 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * @param params
 	 *            terms to be substituted for the respective questionmarks in
 	 *            the query text
+     * @param n the number of solutions to cover
+     * @return an array of up to the the first n binding solutions
 	 */
 	public static final Map<String, Term>[] nSolutions(String text, Term[] params, long n) {
 		return (new Query(text, params)).nSolutions(n);
@@ -780,7 +806,6 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * @return the first solution, if the query has one, as a (possibly empty)
 	 *         Map. If the return value is null, this means that the Query has
 	 *         no solutions.
-	 *         <p>
 	 */
 	public final Map<String, Term> oneSolution() {
 		if (open) {
@@ -806,6 +831,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *
 	 * @param goal
 	 *            the goal of this Query
+	 * @return binding of the first solution
 	 */
 	public static final Map<String, Term> oneSolution(Term goal) {
 		return (new Query(goal)).oneSolution();
@@ -819,6 +845,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *
 	 * @param text
 	 *            a Prolog source text fragment denoting a goal
+	 * @return binding of the first solution
 	 */
 	public static final Map<String, Term> oneSolution(String text) {
 		return (new Query(text)).oneSolution();
@@ -842,6 +869,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * @param params
 	 *            terms to be substituted for the respective questionmarks in
 	 *            the query text
+	 * @return binding of the first solution
 	 */
 	public static final Map<String, Term> oneSolution(String text, Term[] params) {
 		return (new Query(text, params)).oneSolution();
@@ -873,6 +901,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *
 	 * @param goal
 	 *            the goal of this Query
+	 * @return true iff the query can be proved
 	 */
 	public static final boolean hasSolution(Term goal) {
 		return (new Query(goal)).hasSolution();
@@ -886,6 +915,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 *
 	 * @param text
 	 *            the goal of this Query, as Prolog source text
+	 * @return true iff the query can be proved
 	 */
 	public static final boolean hasSolution(String text) {
 		return (new Query(text)).hasSolution();
@@ -909,6 +939,7 @@ public class Query implements Iterable<Map<String, Term>>, Iterator<Map<String,
 	 * @param params
 	 *            terms to be substituted for the respective questionmarks in
 	 *            the query text
+	 * @return true iff the query can be proved
 	 */
 	public static final boolean hasSolution(String text, Term[] params) {
 		return (new Query(text, params)).hasSolution();
