@@ -48,18 +48,54 @@ OPENSSL_zalloc(size_t num)
 #endif
 
 #ifndef HAVE_EVP_MD_CTX_FREE
-static void
+static inline void
 EVP_MD_CTX_free(EVP_MD_CTX *ctx)
 { EVP_MD_CTX_cleanup(ctx);
   OPENSSL_free(ctx);
 }
 
-static EVP_MD_CTX *
+static inline EVP_MD_CTX *
 EVP_MD_CTX_new(void)
 { return OPENSSL_zalloc(sizeof(EVP_MD_CTX));
 }
 #endif
 
+static int
+unify_bytes_hex(term_t t, size_t len, const unsigned char *data)
+{ char tmp[512];
+  char *out, *o;
+  static const char *tohex = "0123456789ABCDEF";
+  const unsigned char *end = data+len;
+  int rc;
+
+  if ( len*2 <= sizeof(tmp) )
+    out = tmp;
+  else if ( !(out = malloc(len*2)) )
+    return PL_resource_error("memory");
+
+  for(o=out ; data < end; data++)
+  { *o++ = tohex[(*data >> 4) & 0xf];
+    *o++ = tohex[(*data >> 0) & 0xf];
+  }
+
+  rc = PL_unify_chars(t, PL_STRING|REP_ISO_LATIN_1, len*2, out);
+  if ( out != tmp )
+    free(out);
+
+  return rc;
+}
+
+static char *
+ssl_strdup(const char *s)
+{
+    char *new = NULL;
+
+    if (s != NULL && (new = malloc(strlen(s)+1)) != NULL) {
+        strcpy(new, s);
+    }
+    return new;
+}
+
 
 /***********************************************************************
  * Warning, error and debug reporting
@@ -140,19 +176,22 @@ ssl_err(char *fmt, ...)
 }
 #endif
 
-#ifdef NEED_SSL_SET_DEBUG
+static int ssl_debug_level = 0;
+
 static int
 ssl_set_debug(int level)
-{ return nbio_debug(level);
+{ int old = ssl_debug_level;
+
+  ssl_debug_level = level;
+  return old;
 }
-#endif
 
 
 static void
 ssl_deb(int level, char *fmt, ...)
 {
 #if DEBUG
-    if ( nbio_debug(-1) >= level )
+    if ( ssl_debug_level >= level )
     { va_list argpoint;
 
       fprintf(stderr, "Debug: ");
@@ -163,6 +202,15 @@ ssl_deb(int level, char *fmt, ...)
 #endif
 }
 
+static inline int
+ssl_missing(const char *feature)
+{ term_t t = PL_new_term_ref();
+
+  return ( PL_put_atom_chars(t, feature) &&
+	   PL_existence_error("ssl_feature", t) );
+
+}
+
 
 /*
  * BIO routines for SSL over streams
@@ -248,7 +296,7 @@ bio_control(BIO* bio, int cmd, long num, void* ptr)
 static int
 bio_create(BIO* bio)
 {
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
    bio->shutdown = 1;
    bio->init = 1;
    bio->num = -1;
@@ -276,7 +324,7 @@ bio_destroy(BIO* bio)
    return 1;
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
 /*
  * Specify the BIO read and write function structures
  */
