@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2015, University of Amsterdam
+    Copyright (c)  2011-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -39,6 +39,7 @@
 #include <math.h>
 #include "libtai/taia.h"
 #include "libtai/caltime.h"
+#include "libtai/leapsecs.h"
 #include <stdio.h>
 #include <ctype.h>
 
@@ -154,15 +155,12 @@ tz_name_as_atom(int dst)
   dst = (dst > 0);			/* 0 or 1 */
 
   if ( !a[dst] )
-  { wchar_t wbuf[256];
-    const char *str = tz_name(dst);
-    size_t n;
+  { const char *str = tz_name(dst);
 
-    if ( (n = mbstowcs(wbuf, str, sizeof(wbuf)/sizeof(wbuf[0])-1)) != (size_t)-1 )
-    { a[dst] = PL_new_atom_wchars(n, wbuf);
-    } else
-    { a[dst] = PL_new_atom(str);
-    }
+    if ( str )
+      a[dst] = PL_new_atom_mbchars(REP_MB, (size_t)-1, str);
+    else
+      a[dst] = PL_new_atom("<unknown>");
   }
 
   return a[dst];
@@ -206,6 +204,8 @@ get_taia(term_t t, struct taia *taia, double *seconds)
     taia->nano  = (long)(fp*1e9);
     taia->atto  = 0L;
 
+    leapsecs_add(&taia->sec, 0);
+
     return TRUE;
   }
 
@@ -406,19 +406,21 @@ cal_ftm(ftm *ftm, int required)
     ct.offset     = -ftm->utcoff / 60;	/* TBD: make libtai speak seconds */
 
     caltime_tai(&ct, &tai);
-    ftm->stamp  = (double)((int64_t)tai.x - TAI_UTC_OFFSET);
-    ftm->stamp -= (double)ct.second;
-    ftm->stamp += ftm->sec;
-    ftm->flags |= HAS_STAMP;
 
     if ( missing & HAS_WYDAY )
     { caltime_utc(&ct, &tai, &ftm->tm.tm_wday, &ftm->tm.tm_yday);
       ftm->flags |= HAS_WYDAY;
     }
+
+    leapsecs_sub(&tai);
+    ftm->stamp  = (double)((int64_t)tai.x - TAI_UTC_OFFSET);
+    ftm->stamp -= (double)ct.second;
+    ftm->stamp += ftm->sec;
+    ftm->flags |= HAS_STAMP;
+
   }
 }
 
-
 static
 PRED_IMPL("stamp_date_time", 3, stamp_date_time, 0)
 { PRED_LD
@@ -441,10 +443,12 @@ PRED_IMPL("stamp_date_time", 3, stamp_date_time, 0)
       { time_t unixt;
 	int64_t ut64;
 	struct tm tm;
+	struct tai tai = taia.sec;
 
 	utcoffset = tz_offset();
 
-	ut64 = taia.sec.x - TAI_UTC_OFFSET;
+	leapsecs_sub(&tai);
+	ut64 = tai.x - TAI_UTC_OFFSET;
 	unixt = (time_t) ut64;
 
 	if ( (int64_t)unixt == ut64 )
@@ -821,14 +825,16 @@ format_time(IOSTREAM *fd, const wchar_t *format, ftm *ftm, int posix)
 	  { int digits = (arg == NOARG ? 6 : arg);
 
 	    if ( digits > 0 )
-	    { double ip, fp;
-	      char fmt[64];
-
+	    { char fmt[64];
+	      char buf[64];
+	      const char *e;
 
 	      cal_ftm(ftm, HAS_STAMP);
-	      fp = modf(ftm->stamp, &ip) * pow(10, digits);
-	      Ssprintf(fmt, "%%0%dlld", digits);
-	      OUTNUMBER(fd, fmt, (long)(fp+0.5));
+	      Ssprintf(fmt, "%%.%df", digits);
+	      Ssprintf(buf, fmt, ftm->stamp);
+	      for(e=buf+strlen(buf); e>buf && e[-1]>='0' && e[-1]<='9'; e--)
+		;
+	      OUTSTR(e);
 	    }
 	    break;
 	  }
@@ -963,8 +969,10 @@ pl_format_time(term_t out, term_t format, term_t time, int posix)
   memset(&tb, 0, sizeof(tb));
   if ( get_taia(time, &taia, &tb.stamp) )
   { double ip;
+    struct tai tai = taia.sec;
 
-    ut64 = taia.sec.x - TAI_UTC_OFFSET;
+    leapsecs_sub(&tai);
+    ut64 = tai.x - TAI_UTC_OFFSET;
     unixt = (time_t) ut64;
 
     if ( (int64_t)unixt == ut64 )
