@@ -340,7 +340,9 @@ memarea_limit(const char *s)
   if ( str_number((unsigned char *)s, &q, &n, FALSE) == NUM_OK &&
        intNumber(&n) )
   { switch((int)*q)
-    { case 'k':
+    { case 0:
+	return (size_t)n.value.i;
+      case 'k':
       case 'K':
 	return (size_t)n.value.i K;
       case 'm':
@@ -649,6 +651,12 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
 	  }
 	} else
 	  return -1;
+      } else if ( (rc=is_bool_opt(s, "threads", &b)) )
+      { if ( rc == TRUE )
+	{ if ( !b )
+	    GD->options.nothreads = TRUE;
+	} else
+	  return -1;
       } else if ( (rc=is_bool_opt(s, "tty", &b)) )
       { if ( rc == TRUE )
 	{ if ( b )
@@ -667,6 +675,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
 #endif
       } else if ( (optval=is_longopt(s, "traditional")) )
       { setTraditional();
+#ifdef HAVE_SIGNAL
       } else if ( (optval=is_longopt(s, "sigalert")) )
       { char *e;
 	long sig = strtol(optval, &e, 10);
@@ -675,6 +684,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
 	  GD->signals.sig_alert = sig;
 	else
 	  return -1;
+#endif
       } else if ( (optval=is_longopt(s, "stack_limit")) )
       { size_t size = memarea_limit(optval);
 
@@ -689,6 +699,8 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft, int compile)
 	  return -1;
 
 	GD->options.tableSpace = size;
+      } else if ( (optval=is_longopt(s, "dump-runtime-variables")) )
+      { GD->options.config = store_string(optval);
       } else if ( !compile )
       { argvleft[argcleft++] = argv[0];
       }
@@ -793,6 +805,8 @@ openResourceDB(int argc, char **argv)
   char *xfile = NULL;
   int flags = (GD->bootsession ? RC_WRONLY|RC_CREATE|RC_TRUNC : RC_RDONLY);
   char tmp[MAXPATHLEN];
+  char plp[MAXPATHLEN];
+  char *exe, *exedir;
   int n;
 
   for(n=0; n<argc-1; n++)
@@ -827,6 +841,13 @@ openResourceDB(int argc, char **argv)
 
   if ( (rc=zip_open_archive(tmp, flags)) )
     return rc;
+  if ( (exe = PrologPath(tmp, plp, sizeof(plp))) &&
+       (exedir = DirName(exe, plp)) &&
+       strlen(exedir)+strlen("/swipl.prc")+1 < MAXPATHLEN )
+  { strcat(exedir, "/swipl.prc");
+    if ( (rc=zip_open_archive(exedir, flags)) )
+      return rc;
+  }
 
   if ( systemDefaults.home )
   { if ( strlen(systemDefaults.home)+1+strlen(BOOTFILE) < MAXPATHLEN )
@@ -901,9 +922,8 @@ PL_initialise(int argc, char **argv)
     argc--; argv++;
     char **argvleft;
 
-    if ( argc == 1 && giveVersionInfo(argv[0]) ) /* -help, -v, etc */
-    { exit(0);
-    }
+    if ( argc == 1 && giveVersionInfo(argv[0]) ) /* --help, --version, etc */
+      exit(0);
 
     for(n=0; n<argc; n++)		/* need to check this first */
     { if ( streq(argv[n], "--" ) )	/* --: terminates argument list */
@@ -947,7 +967,7 @@ PL_initialise(int argc, char **argv)
     return FALSE;
 #ifdef O_PLMT
   aliasThread(PL_thread_self(), ATOM_thread, ATOM_main);
-  enableThreads(TRUE);
+  enableThreads(!GD->options.nothreads);
 #endif
   PL_set_prolog_flag("resource_database", PL_ATOM|FF_READONLY, rcpath);
   initialiseForeign(GD->cmdline.os_argc, /* PL_initialise_hook() functions */
@@ -975,13 +995,12 @@ PL_initialise(int argc, char **argv)
 #endif
       PL_halt(1);
     }
-#ifdef __WINDOWS__
-    PlMessage("Boot compilation has created %s", rcpathcopy);
-#else
-    Sfprintf(Serror,
-	     "Boot compilation has created %s\n", rcpathcopy);
-#endif
-    PL_halt(0);
+
+    predicate_t boot_message2 = PL_predicate("$boot_message", 2, "system");
+    term_t av = PL_new_term_refs(2);
+    PL_halt((PL_put_string_chars(av+0, "Boot compilation has created ~w~n") &&
+	     PL_put_string_chars(av+1, rcpathcopy) &&
+	     PL_call_predicate(NULL, PL_Q_NODEBUG, boot_message2, av)) ? 0 : 1);
   } else
   { IOSTREAM *statefd = SopenZIP(GD->resources.DB, "$prolog/state.qlf", RC_RDONLY);
 
@@ -1007,9 +1026,12 @@ PL_initialise(int argc, char **argv)
 
   DEBUG(MSG_INITIALISE, Sdprintf("Starting Prolog Part of initialisation\n"));
 
-  if ( compile )
+  if ( GD->options.config )
+  { int status = prologToplevel(PL_new_atom("$config")) ? 0 : 1;
+    PL_halt(status);
+    fail;				/* make compiler happy */
+  } else if ( compile )
   { int status = prologToplevel(PL_new_atom("$compile")) ? 0 : 1;
-
     PL_halt(status);
     fail;				/* make compiler happy */
   } else
@@ -1040,7 +1062,7 @@ usage(void)
     "    1) %s [options] prolog-file ... [-- arg ...]\n",
     "    2) %s [options] [-o executable] -c prolog-file ...\n",
     "    3) %s --help     Display this message (also -h)\n",
-    "    4) %s --version  Display version information (also -v)\n",
+    "    4) %s --version  Display version information\n",
     "    4) %s --arch     Display architecture\n",
     "    6) %s --dump-runtime-variables[=format]\n"
     "                        Dump link info in sh(1) format\n",
@@ -1057,6 +1079,7 @@ usage(void)
     "    -O                       Optimised compilation\n",
     "    --tty[=bool]             (Dis)allow tty control\n",
     "    --signals[=bool]         Do (not) modify signal handling\n",
+    "    --threads[=bool]         Do (not) allow for threads\n",
     "    --debug[=bool]           Do (not) generate debug info\n",
     "    --quiet[=bool] (-q)      Do (not) suppress informational messages\n",
     "    --traditional            Disable extensions of version 7\n",
@@ -1115,80 +1138,6 @@ arch(void)
   return TRUE;
 }
 
-#define FMT_SH 1			/* Unix sh: name="value" */
-#define FMT_CMD 2			/* Windows cmd.exe: set name=value */
-
-static void
-printvar(const char *name, const char *value, int format)
-{ switch(format)
-  { case FMT_SH:
-      Sprintf("%s=\"%s\";\n", name, value);
-      break;
-    case FMT_CMD:
-      Sprintf("SET %s=%s\n", name, value);
-      break;
-    default:
-      assert(0);
-  }
-}
-
-
-static int
-runtime_vars(int format)
-{ char *home;
-#ifdef O_XOS
-  char base[MAXPATHLEN];
-#endif
-  char version[20];
-  char *tag = PLVERSION_TAG;
-
-  if ( systemDefaults.home )
-  {
-#ifdef O_XOS
-    if ( format == FMT_CMD )
-    { _xos_os_filename(systemDefaults.home, base, MAXPATHLEN);
-      home = base;
-    } else
-      home = systemDefaults.home;
-#else
-    home = systemDefaults.home;
-#endif
-  } else
-  { home = "<no home>";
-  }
-
-  Ssprintf(version, "%d", PLVERSION);
-
-  printvar("CC",	C_CC, format);
-  printvar("PLBASE",	home, format);
-  printvar("PLARCH",	PLARCH, format);
-  printvar("PLLIBS",	C_LIBS, format);
-  printvar("PLLIB",	C_PLLIB, format);
-  printvar("PLCFLAGS",  C_CFLAGS, format);
-  printvar("PLLDFLAGS", C_LDFLAGS, format);
-#ifdef SO_EXT
-  printvar("PLSOEXT",	SO_EXT, format);
-#endif
-#ifdef SO_PATH
-  printvar("PLSOPATH",	SO_PATH, format);
-#endif
-  printvar("PLVERSION", version, format);
-  if ( tag[0] )
-    printvar("PLVERSIONTAG", tag, format);
-#if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
-  printvar("PLSHARED",	"yes", format);
-#else
-  printvar("PLSHARED",	"no", format);
-#endif
-#ifdef O_PLMT
-  printvar("PLTHREADS", "yes", format);
-#else
-  printvar("PLTHREADS", "no", format);
-#endif
-
-  return TRUE;
-}
-
 
 static int
 giveVersionInfo(const char *a)
@@ -1202,13 +1151,6 @@ giveVersionInfo(const char *a)
   if ( streq(a, "--version") )
     return version();
 
-  if ( streq(a, "--dump-runtime-variables") )
-    return runtime_vars(FMT_SH);
-  if ( streq(a, "--dump-runtime-variables=sh") )
-    return runtime_vars(FMT_SH);
-  if ( streq(a, "--dump-runtime-variables=cmd") )
-    return runtime_vars(FMT_CMD);
-
   return FALSE;
 }
 
