@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam
+    Copyright (c)  1985-2010, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -1488,6 +1488,7 @@ copy_record(Word p, CopyInfo b ARG_LD)
 	    name = ATOM_dict;
 	    break;
 	  default:
+	    name = 0;
 	    assert(0);
 	}
 
@@ -1857,6 +1858,7 @@ PL_recorded_external(const char *rec, term_t t)
 	  fetchAtomW(&b, &a);
 	  break;
 	default:
+	  a = 0;
 	  assert(0);
       }
       rc = PL_put_atom(t, a);
@@ -1915,12 +1917,12 @@ real atom.
 
 bool
 unifyKey(term_t key, word val)
-{ if ( isAtom(val) || isTaggedInt(val) )
+{ GET_LD
+
+  if ( isAtom(val) || isTaggedInt(val) )
   { return _PL_unify_atomic(key, val);
   } else
-  { GET_LD
-
-    return PL_unify_functor(key, (functor_t) val);
+  { return PL_unify_functor(key, (functor_t) val);
   }
 }
 
